<?php
/**
 * Ce fichier contient les fonctions de conversion entre unités, en particulier, les conversions du système métrique
 * vers le système impérial US.
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Lister les 16 directions abrégées en anglais ainsi que la direction variable.
 *
 * @return array Liste des directions abrégées.
 */
function lister_directions() : array {
	static $liste_directions = [
		0  => 'N',
		1  => 'NNE',
		2  => 'NE',
		3  => 'ENE',
		4  => 'E',
		5  => 'ESE',
		6  => 'SE',
		7  => 'SSE',
		8  => 'S',
		9  => 'SSW',
		10 => 'SW',
		11 => 'WSW',
		12 => 'W',
		13 => 'WNW',
		14 => 'NW',
		15 => 'NNW',
		16 => 'N',
		17 => 'V'
	];

	return $liste_directions;
}

/**
 * Convertit un angle en degré en une direction sur 16 valeurs.
 *
 * @param int $angle Angle du vent exprimé en degrés.
 *
 * @return string Direction du vent en abrégée anglais standard selon 16 directions.
 */
function angle2direction(int $angle) : string {
	static $liste_directions = [];

	if (!$liste_directions) {
		$liste_directions = lister_directions();
	}

	$direction = $liste_directions[round($angle / 22.5) % 16];

	return $direction;
}

/**
 * Convertit un indice UV normalisé en une chaine correspondant à un niveau de risque.
 *
 * @param int $indice_uv Entier représentant l'indice UV
 *
 * @return string Chaine représentant le risque lié à l'indice UV. Cette chaine permet de calculer
 *                l'item de langue du risque dans la langue requise.
 */
function indice2risque_uv(int $indice_uv) : string {
	$risque_uv = '';
	if (is_float($indice_uv)) {
		if ($indice_uv >= 11) {
			$risque_uv = 'extreme';
		} elseif ($indice_uv >= 8) {
			$risque_uv = 'tres_eleve';
		} elseif ($indice_uv >= 6) {
			$risque_uv = 'eleve';
		} elseif ($indice_uv >= 3) {
			$risque_uv = 'modere';
		} elseif ($indice_uv >= 0) {
			$risque_uv = 'faible';
		}
	}

	return $risque_uv;
}

/**
 * Convertit des kilomètres en miles.
 *
 * @param float $kilometre La valeur réelle en kilomètres.
 *
 * @return float La valeur réelle correspondante convertie en miles
 */
function kilometre2mile(float $kilometre) : float {
	return 0.6215 * $kilometre;
}

/**
 * Convertit des températures celsius en farenheit.
 *
 * @param int $celsius La valeur réelle en degrés celsius.
 *
 * @return float La valeur réelle correspondante convertie en farenheit.
 */
function celsius2farenheit(int $celsius) : float {
	return $celsius * 9 / 5 + 32;
}

/**
 * Convertit des millimètres en pouces.
 *
 * @param float $millimetre La valeur réelle en millimètres
 *
 * @return float La valeur réelle correspondante convertie en pouces.
 */
function millimetre2inch(float $millimetre) : float {
	return $millimetre / 25.4;
}

/**
 * Convertit des pressions millibar en pouces.
 *
 * @param float $millibar La valeur réelle en millibars
 *
 * @return float La valeur réelle correspondante convertie en pouces.
 */
function millibar2inch(float $millibar) : float {
	return $millibar / 33.86;
}

/**
 * Calcule la température ressentie (refroidissement éolien) en degrés celsius.
 *
 * Le calcul n'a de sens que pour des températures réelles supérieures à -50°C et inférieures à 10°C.
 * Au-delà de ces valeurs, la fonction renvoie la température réelle fournie en entrée.
 *
 * @param float $temperature  Température réelle mesurée en celsius.
 * @param float $vitesse_vent Vitesse du vent en kilomètre par heure.
 *
 * @return float Température ressentie arrondie en entier et exprimée en degrés celsius.
 */
function temperature2ressenti(float $temperature, float $vitesse_vent) : float {
	if (($temperature >= -50) and ($temperature <= 10)) {
		if ($vitesse_vent > 4.8) {
			$ressenti = 13.12 + 0.6215 * $temperature + (0.3965 * $temperature - 11.37) * pow($vitesse_vent, 0.16);
		} else {
			$ressenti = $temperature + 0.2 * (0.1345 * $temperature - 1.59) * $vitesse_vent;
		}
		$ressenti = round($ressenti, 0);
	} else {
		$ressenti = $temperature;
	}

	return $ressenti;
}

/**
 * Convertit en kilomètres une valeur en mètres.
 *
 * @param int $metre La valeur entière en mètres
 *
 * @return float La valeur correspondante convertie en kilomètres.
 */
function metre2kilometre(int $metre) : float {
	return $metre / 1000;
}

/**
 * Convertit en kilomètres par heure une valeur en mètres par seconde.
 *
 * @param float $vitesse_vent Vitesse du vent en mètres par seconde.
 *
 * @return float La valeur correspondante convertie en kilomètres par heure.
 */
function metre_seconde2kilometre_heure(float $vitesse_vent) : float {
	return ($vitesse_vent * 3600) / 1000;
}
